package wiki

import "strings"

func NormalizeWikiTitle(title string) string {
	//TODO
	return strings.Replace(title, " ", "_", -1)
}

//ParseWikiText small WikiText parser that extracts text, Templates, and its arguments/parameters
func ParseWikiText(text string) (result []interface{}) {
	index := 0

	for {
		templateIndex := strings.Index(text[index:], "{{")
		if templateIndex == -1 {
			t := strings.TrimSpace(text[index:])
			if len(t) > 0 {
				result = append(result, t)
			}
			break
		} else {
			t := strings.TrimSpace(text[index : index+templateIndex])
			if len(t) > 0 {
				result = append(result, t)
			}
			var tpl *Template
			index, tpl = ParseTemplate(text, index+templateIndex+2, 0)
			if tpl != nil {
				result = append(result, tpl)
			}
		}
	}

	return
}

func ParseTemplate(text string, index int, depth int) (i int, template *Template) {

	var c byte
	lastToken := index

	var key string

	addValue := func() int {
		if lastToken < len(text) && i-lastToken > 0 {
			t := strings.TrimSpace(text[lastToken:i])
			if len(t) > 0 {
				if template == nil {
					template = NewTemplate(t)
				} else {
					if key == "" {
						template.AddParameterUnkeyed(t)
					} else {
						template.AddParameter(key, t)
					}
				}
			}

			return len(t)
		}

		return 0
	}
	addKey := func() {
		if lastToken < len(text) && i-lastToken > 0 {
			t := strings.TrimSpace(text[lastToken:i])
			if len(t) > 0 {
				key = t
			}
		}
	}
	for i = index; i < len(text); i++ {
		c = text[i]

		if c == '}' && i < len(text)-1 && text[i+1] == '}' {
			addValue()
			i += 2
			break
		} else if c == '{' && i < len(text)-1 && text[i+1] == '{' {
			addValue()
			var tpl *Template
			i, tpl = ParseTemplate(text, i+2, depth+1)
			if tpl != nil {
				if key == "" {
					template.AddParameterUnkeyed(tpl)
				} else {
					template.AddParameter(key, tpl)
				}
			}
			lastToken = i
		} else if c == '|' {
			addValue()
			lastToken = i + 1
			key = ""
		} else if c == '\n' {
			addValue()
			lastToken = i + 1
		} else if c == '=' {
			if key == "" {
				addKey()
				lastToken = i + 1
			}
		}
	}

	return
}
